@@ -21,10 +21,10 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -67))
+                ((*default* . -68))
+                ((*default* . -74))
                 ((*default* . -73))
                 ((*default* . -72))
-                ((*default* . -71))
                 ((*default* . *error*)
                  (IDENT . 19)
                  (STRING . 17)
@@ -107,13 +107,13 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -70))
-                ((*default* . -68))
-                ((*default* . -65))
-                ((*default* . -64))
-                ((*default* . -63))
+                ((*default* . -71))
                 ((*default* . -69))
                 ((*default* . -66))
+                ((*default* . -65))
+                ((*default* . -64))
+                ((*default* . -70))
+                ((*default* . -67))
                 ((*default* . *error*)
                  (IDENT . 19)
                  (STRING . 17)
@@ -133,8 +133,8 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -62))
-                ((*default* . -39)
+                ((*default* . -63))
+                ((*default* . -40)
                  (IDENT . 19)
                  (STRING . 17)
                  (HASH . 16)
@@ -150,11 +150,11 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -86) (S . 29))
+                ((*default* . -87) (S . 29))
                 ((*default* . *error*) (LBRACE . 9))
-                ((*default* . -8))
-                ((*default* . -35) (S . 29))
-                ((*default* . -9))
+                ((*default* . -9) (ATKEYWORD . 18))
+                ((*default* . -36) (S . 29))
+                ((*default* . -10))
                 ((*default* . -7))
                 ((*default* . -2)
                  (IDENT . 19)
@@ -176,9 +176,9 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . *error*) (*eoi* . 65))
-                ((*default* . -90) (S . 29))
-                ((*default* . -74))
+                ((*default* . *error*) (*eoi* . 66))
+                ((*default* . -91) (S . 29))
+                ((*default* . -75))
                 ((*default* . *error*)
                  (IDENT . 19)
                  (STRING . 17)
@@ -189,15 +189,15 @@
                  (URI . 12)
                  (UNICODE-RANGE . 11)
                  (LPAREN . 8)
-                 (RPAREN . 67)
+                 (RPAREN . 68)
                  (LBRACK . 7)
                  (FUNCTION . 5)
                  (EXACTMATCH . 4)
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . *error*) (RPAREN . 69))
-                ((*default* . -88)
+                ((*default* . *error*) (RPAREN . 70))
+                ((*default* . -89)
                  (IDENT . 19)
                  (STRING . 17)
                  (HASH . 16)
@@ -213,7 +213,7 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -82))
+                ((*default* . -83))
                 ((*default* . *error*)
                  (IDENT . 19)
                  (STRING . 17)
@@ -225,14 +225,14 @@
                  (UNICODE-RANGE . 11)
                  (LPAREN . 8)
                  (LBRACK . 7)
-                 (RBRACK . 71)
+                 (RBRACK . 72)
                  (FUNCTION . 5)
                  (EXACTMATCH . 4)
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . *error*) (RBRACK . 73))
-                ((*default* . -78))
+                ((*default* . *error*) (RBRACK . 74))
+                ((*default* . -79))
                 ((*default* . *error*)
                  (IDENT . 19)
                  (STRING . 17)
@@ -243,37 +243,37 @@
                  (URI . 12)
                  (UNICODE-RANGE . 11)
                  (LPAREN . 8)
-                 (RPAREN . 74)
+                 (RPAREN . 75)
                  (LBRACK . 7)
                  (FUNCTION . 5)
                  (EXACTMATCH . 4)
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . *error*) (RPAREN . 76))
+                ((*default* . *error*) (RPAREN . 77))
                 ((*default* . *error*) (IDENT . 43))
-                ((*default* . -29))
-                ((*default* . -50) (S . 29))
-                ((*default* . -47))
+                ((*default* . -30))
+                ((*default* . -51) (S . 29))
+                ((*default* . -48))
                 ((*default* . *error*)
                  (IDENT . 43)
                  (SEMICOLON . 42)
                  (RBRACE . 41)
                  (DELIM . 40))
-                ((*default* . *error*) (RBRACE . 80))
-                ((*default* . -54) (SEMICOLON . 42))
-                ((*default* . -52) (IDENT . 43) (DELIM . 40))
-                ((*default* . *error*) (S . 29) (DELIM . 83))
+                ((*default* . *error*) (RBRACE . 81))
+                ((*default* . -55) (SEMICOLON . 42))
+                ((*default* . -53) (IDENT . 43) (DELIM . 40))
+                ((*default* . *error*) (S . 29) (DELIM . 84))
                 ((*default* . *error*)
                  (SEMICOLON . 42)
-                 (RBRACE . 85))
-                ((*default* . -46))
-                ((*default* . -33))
+                 (RBRACE . 86))
+                ((*default* . -47))
+                ((*default* . -34))
                 ((*default* . -4))
-                ((*default* . *error*) (CDC . 87))
+                ((*default* . *error*) (CDC . 88))
                 ((*default* . *error*)
                  (IDENT . 19)
-                 (ATKEYWORD . 89)
+                 (ATKEYWORD . 90)
                  (STRING . 17)
                  (HASH . 16)
                  (NUMBER . 15)
@@ -283,7 +283,7 @@
                  (UNICODE-RANGE . 11)
                  (SEMICOLON . 42)
                  (LBRACE . 54)
-                 (RBRACE . 88)
+                 (RBRACE . 89)
                  (LPAREN . 8)
                  (LBRACK . 7)
                  (S . 29)
@@ -313,35 +313,36 @@
                 ((*default* . *error*)
                  (SEMICOLON . 42)
                  (LBRACE . 54))
+                ((*default* . -50))
+                ((*default* . -11))
                 ((*default* . -49))
-                ((*default* . -10))
-                ((*default* . -48))
-                ((*default* . -40))
-                ((*default* . -87))
-                ((*default* . -37) (S . 29))
-                ((*default* . -36))
+                ((*default* . -41))
+                ((*default* . -88))
+                ((*default* . -38) (S . 29))
+                ((*default* . -8))
+                ((*default* . -37))
                 ((*default* . -3))
                 ((*default* . -1) (*eoi* . accept))
-                ((*default* . -91))
-                ((*default* . -75))
-                ((*default* . *error*) (RPAREN . 100))
+                ((*default* . -92))
                 ((*default* . -76))
-                ((*default* . -89))
-                ((*default* . -83))
-                ((*default* . *error*) (RBRACK . 101))
+                ((*default* . *error*) (RPAREN . 101))
+                ((*default* . -77))
+                ((*default* . -90))
                 ((*default* . -84))
-                ((*default* . -79))
-                ((*default* . *error*) (RPAREN . 102))
+                ((*default* . *error*) (RBRACK . 102))
+                ((*default* . -85))
                 ((*default* . -80))
-                ((*default* . -45))
-                ((*default* . -51))
-                ((*default* . -34))
-                ((*default* . -30))
-                ((*default* . -55))
-                ((*default* . -53))
+                ((*default* . *error*) (RPAREN . 103))
+                ((*default* . -81))
+                ((*default* . -46))
+                ((*default* . -52))
+                ((*default* . -35))
+                ((*default* . -31))
+                ((*default* . -56))
+                ((*default* . -54))
                 ((*default* . *error*)
                  (IDENT . 19)
-                 (ATKEYWORD . 103)
+                 (ATKEYWORD . 18)
                  (STRING . 17)
                  (HASH . 16)
                  (NUMBER . 15)
@@ -358,15 +359,15 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . *error*) (DELIM . 109))
-                ((*default* . -31))
-                ((*default* . *error*) (RBRACE . 110))
+                ((*default* . *error*) (DELIM . 110))
+                ((*default* . -32))
+                ((*default* . *error*) (RBRACE . 111))
                 ((*default* . -5))
-                ((*default* . -14) (S . 29))
-                ((*default* . -26) (S . 29))
+                ((*default* . -15) (S . 29))
+                ((*default* . -27) (S . 29))
                 ((*default* . *error*)
                  (IDENT . 19)
-                 (ATKEYWORD . 89)
+                 (ATKEYWORD . 90)
                  (STRING . 17)
                  (HASH . 16)
                  (NUMBER . 15)
@@ -376,7 +377,7 @@
                  (UNICODE-RANGE . 11)
                  (SEMICOLON . 42)
                  (LBRACE . 54)
-                 (RBRACE . 113)
+                 (RBRACE . 114)
                  (LPAREN . 8)
                  (LBRACK . 7)
                  (FUNCTION . 5)
@@ -384,111 +385,60 @@
                  (INCLUDES . 3)
                  (DASHMATCH . 2)
                  (DELIM . 1))
-                ((*default* . -24))
-                ((*default* . -28))
-                ((*default* . -22)
-                 (IDENT . 19)
-                 (ATKEYWORD . 89)
-                 (STRING . 17)
-                 (HASH . 16)
-                 (NUMBER . 15)
-                 (PERCENTAGE . 14)
-                 (DIMENSION . 13)
-                 (URI . 12)
-                 (UNICODE-RANGE . 11)
-                 (SEMICOLON . 42)
-                 (LBRACE . 54)
-                 (LPAREN . 8)
-                 (LBRACK . 7)
-                 (FUNCTION . 5)
-                 (EXACTMATCH . 4)
-                 (INCLUDES . 3)
-                 (DASHMATCH . 2)
-                 (DELIM . 1))
-                ((*default* . *error*) (RBRACE . 116))
                 ((*default* . -25))
+                ((*default* . -29))
+                ((*default* . -23)
+                 (IDENT . 19)
+                 (ATKEYWORD . 90)
+                 (STRING . 17)
+                 (HASH . 16)
+                 (NUMBER . 15)
+                 (PERCENTAGE . 14)
+                 (DIMENSION . 13)
+                 (URI . 12)
+                 (UNICODE-RANGE . 11)
+                 (SEMICOLON . 42)
+                 (LBRACE . 54)
+                 (LPAREN . 8)
+                 (LBRACK . 7)
+                 (FUNCTION . 5)
+                 (EXACTMATCH . 4)
+                 (INCLUDES . 3)
+                 (DASHMATCH . 2)
+                 (DELIM . 1))
+                ((*default* . *error*) (RBRACE . 117))
+                ((*default* . -26))
                 ((*default* . *error*)
                  (SEMICOLON . 42)
                  (LBRACE . 54))
-                ((*default* . -11))
                 ((*default* . -12))
-                ((*default* . -38))
-                ((*default* . -77))
-                ((*default* . -85))
-                ((*default* . -81))
-                ((*default* . -60) (S . 29))
-                ((*default* . *error*)
-                 (IDENT . 19)
-                 (ATKEYWORD . 103)
-                 (STRING . 17)
-                 (HASH . 16)
-                 (NUMBER . 15)
-                 (PERCENTAGE . 14)
-                 (DIMENSION . 13)
-                 (URI . 12)
-                 (UNICODE-RANGE . 11)
-                 (LBRACE . 54)
-                 (LPAREN . 8)
-                 (LBRACK . 7)
-                 (FUNCTION . 5)
-                 (EXACTMATCH . 4)
-                 (INCLUDES . 3)
-                 (DASHMATCH . 2)
-                 (DELIM . 1))
-                ((*default* . -58))
-                ((*default* . -56)
-                 (IDENT . 19)
-                 (ATKEYWORD . 103)
-                 (STRING . 17)
-                 (HASH . 16)
-                 (NUMBER . 15)
-                 (PERCENTAGE . 14)
-                 (DIMENSION . 13)
-                 (URI . 12)
-                 (UNICODE-RANGE . 11)
-                 (LBRACE . 54)
-                 (LPAREN . 8)
-                 (LBRACK . 7)
-                 (FUNCTION . 5)
-                 (EXACTMATCH . 4)
-                 (INCLUDES . 3)
-                 (DASHMATCH . 2)
-                 (DELIM . 1))
-                ((*default* . -41))
-                ((*default* . -59))
-                ((*default* . *error*)
-                 (IDENT . 19)
-                 (ATKEYWORD . 103)
-                 (STRING . 17)
-                 (HASH . 16)
-                 (NUMBER . 15)
-                 (PERCENTAGE . 14)
-                 (DIMENSION . 13)
-                 (URI . 12)
-                 (UNICODE-RANGE . 11)
-                 (LBRACE . 54)
-                 (LPAREN . 8)
-                 (LBRACK . 7)
-                 (S . 29)
-                 (FUNCTION . 5)
-                 (EXACTMATCH . 4)
-                 (INCLUDES . 3)
-                 (DASHMATCH . 2)
-                 (DELIM . 1))
-                ((*default* . -32))
-                ((*default* . -15))
-                ((*default* . -27))
-                ((*default* . -16) (S . 29))
-                ((*default* . *error*) (RBRACE . 124))
-                ((*default* . -23))
-                ((*default* . -18) (S . 29))
                 ((*default* . -13))
-                ((*default* . -61))
-                ((*default* . -43))
-                ((*default* . -57))
+                ((*default* . -39))
+                ((*default* . -78))
+                ((*default* . -86))
+                ((*default* . -82))
                 ((*default* . *error*)
                  (IDENT . 19)
-                 (ATKEYWORD . 103)
+                 (ATKEYWORD . 18)
+                 (STRING . 17)
+                 (HASH . 16)
+                 (NUMBER . 15)
+                 (PERCENTAGE . 14)
+                 (DIMENSION . 13)
+                 (URI . 12)
+                 (UNICODE-RANGE . 11)
+                 (LBRACE . 54)
+                 (LPAREN . 8)
+                 (LBRACK . 7)
+                 (FUNCTION . 5)
+                 (EXACTMATCH . 4)
+                 (INCLUDES . 3)
+                 (DASHMATCH . 2)
+                 (DELIM . 1))
+                ((*default* . -59))
+                ((*default* . -57)
+                 (IDENT . 19)
+                 (ATKEYWORD . 18)
                  (STRING . 17)
                  (HASH . 16)
                  (NUMBER . 15)
@@ -505,11 +455,62 @@
                  (DASHMATCH . 2)
                  (DELIM . 1))
                 ((*default* . -42))
-                ((*default* . -17))
+                ((*default* . -60))
+                ((*default* . -61) (S . 29))
+                ((*default* . *error*)
+                 (IDENT . 19)
+                 (ATKEYWORD . 18)
+                 (STRING . 17)
+                 (HASH . 16)
+                 (NUMBER . 15)
+                 (PERCENTAGE . 14)
+                 (DIMENSION . 13)
+                 (URI . 12)
+                 (UNICODE-RANGE . 11)
+                 (LBRACE . 54)
+                 (LPAREN . 8)
+                 (LBRACK . 7)
+                 (S . 29)
+                 (FUNCTION . 5)
+                 (EXACTMATCH . 4)
+                 (INCLUDES . 3)
+                 (DASHMATCH . 2)
+                 (DELIM . 1))
+                ((*default* . -33))
+                ((*default* . -16))
+                ((*default* . -28))
+                ((*default* . -17) (S . 29))
+                ((*default* . *error*) (RBRACE . 125))
+                ((*default* . -24))
                 ((*default* . -19) (S . 29))
-                ((*default* . -20))
+                ((*default* . -14))
                 ((*default* . -44))
-                ((*default* . -21))))
+                ((*default* . -58))
+                ((*default* . -62))
+                ((*default* . *error*)
+                 (IDENT . 19)
+                 (ATKEYWORD . 18)
+                 (STRING . 17)
+                 (HASH . 16)
+                 (NUMBER . 15)
+                 (PERCENTAGE . 14)
+                 (DIMENSION . 13)
+                 (URI . 12)
+                 (UNICODE-RANGE . 11)
+                 (LBRACE . 54)
+                 (LPAREN . 8)
+                 (LBRACK . 7)
+                 (FUNCTION . 5)
+                 (EXACTMATCH . 4)
+                 (INCLUDES . 3)
+                 (DASHMATCH . 2)
+                 (DELIM . 1))
+                ((*default* . -43))
+                ((*default* . -18))
+                ((*default* . -20) (S . 29))
+                ((*default* . -21))
+                ((*default* . -45))
+                ((*default* . -22))))
            (___gtable
              (vector
                '((22 . 20)
@@ -563,8 +564,8 @@
                '((22 . 20) (21 . 21) (11 . 60))
                '((24 . 61))
                '((9 . 62))
-               '()
-               '((24 . 63))
+               '((4 . 63))
+               '((24 . 64))
                '()
                '()
                '((22 . 20)
@@ -575,22 +576,22 @@
                  (4 . 25)
                  (3 . 26)
                  (2 . 27)
-                 (1 . 64))
+                 (1 . 65))
                '()
-               '((24 . 66))
+               '((24 . 67))
                '()
-               '((23 . 68) (22 . 33) (21 . 21))
+               '((23 . 69) (22 . 33) (21 . 21))
                '()
-               '((23 . 70) (22 . 33) (21 . 21))
+               '((23 . 71) (22 . 33) (21 . 21))
                '()
-               '((23 . 72) (22 . 33) (21 . 21))
+               '((23 . 73) (22 . 33) (21 . 21))
                '()
                '()
-               '((23 . 75) (22 . 33) (21 . 21))
+               '((23 . 76) (22 . 33) (21 . 21))
                '()
-               '((14 . 48) (12 . 77))
+               '((14 . 48) (12 . 78))
                '()
-               '((24 . 78))
+               '((24 . 79))
                '()
                '((18 . 45)
                  (17 . 46)
@@ -598,36 +599,37 @@
                  (14 . 48)
                  (13 . 49)
                  (12 . 50)
-                 (8 . 79))
+                 (8 . 80))
                '()
-               '((18 . 81) (17 . 46) (16 . 47))
-               '((14 . 48) (13 . 82) (12 . 50))
-               '((24 . 84))
-               '((18 . 86) (17 . 46) (16 . 47))
+               '((18 . 82) (17 . 46) (16 . 47))
+               '((14 . 48) (13 . 83) (12 . 50))
+               '((24 . 85))
+               '((18 . 87) (17 . 46) (16 . 47))
                '()
                '()
                '()
                '()
-               '((24 . 90)
-                 (22 . 91)
+               '((24 . 91)
+                 (22 . 92)
                  (21 . 21)
-                 (16 . 92)
-                 (7 . 93)
-                 (6 . 94)
-                 (5 . 95))
-               '((23 . 96)
+                 (16 . 93)
+                 (7 . 94)
+                 (6 . 95)
+                 (5 . 96))
+               '((23 . 97)
                  (22 . 33)
                  (21 . 21)
                  (16 . 57)
-                 (15 . 97)
+                 (15 . 98)
                  (5 . 59))
-               '((16 . 57) (15 . 98) (5 . 59))
+               '((16 . 57) (15 . 99) (5 . 59))
                '()
                '()
                '()
                '()
                '()
-               '((24 . 99))
+               '((24 . 100))
+               '()
                '()
                '()
                '()
@@ -653,63 +655,67 @@
                  (21 . 21)
                  (20 . 106)
                  (19 . 107)
-                 (5 . 108))
+                 (5 . 108)
+                 (4 . 109))
                '()
                '()
                '()
                '()
-               '((24 . 111))
                '((24 . 112))
-               '((22 . 91)
+               '((24 . 113))
+               '((22 . 92)
                  (21 . 21)
-                 (16 . 92)
-                 (7 . 93)
-                 (6 . 114)
-                 (5 . 95))
-               '()
-               '()
-               '((22 . 91)
-                 (21 . 21)
-                 (16 . 92)
-                 (7 . 93)
+                 (16 . 93)
+                 (7 . 94)
                  (6 . 115)
-                 (5 . 95))
+                 (5 . 96))
                '()
                '()
-               '((16 . 57) (15 . 117) (5 . 59))
+               '((22 . 92)
+                 (21 . 21)
+                 (16 . 93)
+                 (7 . 94)
+                 (6 . 116)
+                 (5 . 96))
+               '()
+               '()
+               '((16 . 57) (15 . 118) (5 . 59))
                '()
                '()
                '()
                '()
                '()
                '()
-               '((24 . 118))
                '((22 . 105)
                  (21 . 21)
                  (20 . 106)
                  (19 . 119)
-                 (5 . 108))
+                 (5 . 108)
+                 (4 . 109))
                '()
                '((22 . 105)
                  (21 . 21)
                  (20 . 106)
                  (19 . 120)
-                 (5 . 108))
+                 (5 . 108)
+                 (4 . 109))
                '()
                '()
-               '((24 . 121)
+               '((24 . 121))
+               '((24 . 122)
                  (22 . 105)
                  (21 . 21)
                  (20 . 106)
-                 (19 . 122)
-                 (5 . 108))
+                 (19 . 123)
+                 (5 . 108)
+                 (4 . 109))
                '()
                '()
                '()
-               '((24 . 123))
+               '((24 . 124))
                '()
                '()
-               '((24 . 125))
+               '((24 . 126))
                '()
                '()
                '()
@@ -717,11 +723,12 @@
                '((22 . 105)
                  (21 . 21)
                  (20 . 106)
-                 (19 . 126)
-                 (5 . 108))
+                 (19 . 127)
+                 (5 . 108)
+                 (4 . 109))
                '()
                '()
-               '((24 . 127))
+               '((24 . 128))
                '()
                '()
                '()))
@@ -794,7 +801,7 @@
                      (- ___sp 2)
                      1
                      ___goto-table
-                     (list $1)
+                     (if (null? $1) $1 (list $1))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -845,6 +852,16 @@
                      ___goto-table
                      $1
                      ___k)))
+               (lambda (___stack ___sp ___goto-table ___k)
+                 (let* (($2 (vector-ref ___stack (- ___sp 1)))
+                        ($1 (vector-ref ___stack (- ___sp 3))))
+                   (___push
+                     ___stack
+                     (- ___sp 4)
+                     3
+                     ___goto-table
+                     $1
+                     ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($1 (vector-ref ___stack (- ___sp 1))))
                    (___push
@@ -861,7 +878,18 @@
                      (- ___sp 2)
                      3
                      ___goto-table
-                     $1
+                     (if (and (equal? (car $1) "@import")
+                              (> (length $1) 1)
+                              (list? (cadr $1))
+                              (> (length (cadr $1)) 0)
+                              (string? (caadr $1)))
+                       (let* ((u (caadr $1))
+                              (uri (if (and (> (string-length u) 9)
+                                            (equal? (substring u 1 4) "url("))
+                                     (substring u 7 (- (string-length u) 3))
+                                     (substring u 1 (- (string-length u) 1)))))
+                         (list (quote @import) uri))
+                       (list))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -871,7 +899,7 @@
                      (- ___sp 4)
                      4
                      ___goto-table
-                     $1
+                     (cons $1 $2)
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($3 (vector-ref ___stack (- ___sp 1)))
@@ -882,7 +910,7 @@
                      (- ___sp 6)
                      4
                      ___goto-table
-                     $1
+                     (cons $1 $3)
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($3 (vector-ref ___stack (- ___sp 1)))
@@ -893,7 +921,7 @@
                      (- ___sp 6)
                      4
                      ___goto-table
-                     $1
+                     (cons $1 (cons $2 $3))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($4 (vector-ref ___stack (- ___sp 1)))
@@ -905,7 +933,7 @@
                      (- ___sp 8)
                      4
                      ___goto-table
-                     $1
+                     (cons $1 (cons $3 $4))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -1081,7 +1109,7 @@
                      (- ___sp 4)
                      8
                      ___goto-table
-                     $1
+                     (list $1)
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -1102,7 +1130,9 @@
                      (- ___sp 6)
                      8
                      ___goto-table
-                     (cons $1 $2)
+                     (cond ((null? $2) (list $1))
+                           ((list? (car $2)) (cons $1 $2))
+                           (else (cons $1 (list $2))))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -1153,7 +1183,9 @@
                      10
                      ___goto-table
                      (let ((m (merge-selectors $1)))
-                       (begin (validate-selector m) (list m $2)))
+                       (if (not (false-if-exception (validate-selector m)))
+                         (list)
+                         (list m $2)))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($3 (vector-ref ___stack (- ___sp 1)))
@@ -1165,7 +1197,9 @@
                      10
                      ___goto-table
                      (let ((m (merge-selectors $1)))
-                       (begin (validate-selector m) (list m $2)))
+                       (if (not (false-if-exception (validate-selector m)))
+                         (list)
+                         (list m $2)))
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($1 (vector-ref ___stack (- ___sp 1))))
@@ -1385,7 +1419,7 @@
                      (- ___sp 2)
                      20
                      ___goto-table
-                     $1
+                     ""
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($2 (vector-ref ___stack (- ___sp 1)))
@@ -1395,7 +1429,7 @@
                      (- ___sp 4)
                      20
                      ___goto-table
-                     $1
+                     ""
                      ___k)))
                (lambda (___stack ___sp ___goto-table ___k)
                  (let* (($1 (vector-ref ___stack (- ___sp 1))))
