@@ -38,6 +38,7 @@
 	    scss:set-id-handler!
 	    scss:set-pseudo-class-handler!
 	    scss:set-pseudo-element-handler!
+	    scss:set-uri-import-handler!
 
 	    scss:stylesheet?
 	    scss:cascade?
@@ -151,7 +152,8 @@
 			    "(top)|(center)|(bottom)")
 	(exact-match "inherit"))))
 
-;; NOTE: Changes to this regex will require updates to the selection functions!
+;; NOTE: Changes to this regex will require updates to the selection 
+;; functions!
 
 (define attr-sel-regexc 
   (make-regexp (string-append "\\[" ident-regex "(([~|]?=)(" ident-regex "|" 
@@ -160,9 +162,10 @@
 (define color-table
   '(("maroon" "#800000") ("red" "#ff0000") ("orange" "#ffa500") 
     ("yellow" "#ffff00") ("olive" "#808000") ("purple" "#800080")
-    ("fuchsia" "#ffffff") ("lime" "#00ff00") ("green" "#008000") 
-    ("navy" "#000080") ("blue" "#0000ff") ("aqua" "#00ffff") ("teal" "#008080")
-    ("black" "#000000") ("silver" "#c0c0c0") ("gray" "#808080")))
+    ("fuchsia" "#ff00ff") ("white" "#ffffff") ("lime" "#00ff00") 
+    ("green" "#008000") ("navy" "#000080") ("blue" "#0000ff") 
+    ("aqua" "#00ffff") ("teal" "#008080") ("black" "#000000") 
+    ("silver" "#c0c0c0") ("gray" "#808080")))
 
 (define exact-match
   (lambda (pattern str)
@@ -221,7 +224,7 @@
     ("border-collapse" ,(matchf "(collapse)|(separate)|(inherit)")
      ("separate") #t)
     ("border-color" ,(matchf "((color)|(transparent)){1,4}|(inherit)") 
-     (color color color color) #f)
+     (color color color color) "color")
     ("border-spacing"
      ,(matchf (string-append "(" length-regex "\\w+" length-regex 
 			     "?)|(inherit)")) ("0") #t)
@@ -237,7 +240,7 @@
      ("medium" "none" color) #f)
     (,(string-append "(border-top-color)|(border-right-color)|"
 		     "(border-left-color)|(border-bottom-color)")
-     ,(matchf pcolor-regex) (color) #f)
+     ,(matchf pcolor-regex) (color) "color")
     (,(string-append "(border-top-style)|(border-right-style)|"
 		     "(border-left-style)|(border-bottom-style)")
      ,(matchf (string-append border-style-regex "|(inherit)")) ("none") #f)
@@ -261,7 +264,8 @@
     ("clear" ,(matchf "(none)|(left)|(right)|(both)|(inherit)") ("none") #f)
     ("clip" 
      ,(matchf (string-append "(rect\\((" length-regex "|(auto))(,\\w(" 
-			     length-regex "|(auto))){3}\\))|(auto)|(inherit)"))
+			     length-regex "|(auto))){3}\\))|(auto)|"
+			     "(inherit)"))
      ("auto") #f)
     ("color" ,(matchf pcolor-regex) ("white") #t)
     ("content" 
@@ -454,9 +458,14 @@
     ("z-index" ,(matchf (string-append "(auto)|" integer-regex "|(inherit)")) 
      ("auto") #f)))
 
-(define sxml-node? 
-  (lambda (x) (and (list? x) (> (length x) 1) (symbol? (car x)))))
-(define sxml-doc? (lambda (x) (and (sxml-node? x) (eq? (car x) '*TOP*))))
+(define node?
+  (lambda (x)
+    (and (list? x) 
+	 (> (length x) 1) 
+	 (or (symbol? (car x)) 
+	     (string? (car x))))))
+(define sxml-node? (lambda (x) (and (node? x) (not (eq? (car x) '*TOP*)))))
+(define sxml-doc? (lambda (x) (and (node? x) (eq? (car x) '*TOP*))))
 (define get-sxml-parent
   (lambda (doc node)
     (if (memq node doc) doc (let ((c (filter list? (cdr doc))))
@@ -466,16 +475,19 @@
 			      #f))))
 (define sxml-node-name
   (lambda (node)
-    (let* ((str (symbol->string (car node)))
-	   (ri (string-rindex str #\:)))
-      (if (not ri) str (substring str ri)))))      
+    (let* ((str (if (string? (car node)) '() (symbol->string (car node))))
+	   (ri (if (string? str) (string-rindex str #\:) #f)))
+      (if (not ri) str (substring str (+ ri 1))))))
 (define sxml-attr-val
   (lambda (node name)
-    (let ((attrs (find (lambda (x) (and (list? x) (eq? (car x) '@)))
-		       (cdr node))))
-      (if attrs (find (lambda (x) 
-			(and (list? x) (equal? (sxml-node-name x) name))) 
-		      (cdr attrs)) #f))))			 
+    (if (string? (car node)) 
+	#f
+	(let ((attrs (find (lambda (x) (and (list? x) (eq? (car x) '@)))
+			   (cdr node))))
+	  (if attrs (find (lambda (x) 
+			    (and (list? x) (equal? (sxml-node-name x) name))) 
+			  (cdr attrs)) 
+	      #f)))))			 
 (define scss:set-sxml-parent-function!
   (lambda (proc) 
     (verify-arg-types "scss:set-sxml-parent-function!" 
@@ -514,6 +526,15 @@
 		      1)
     (set! internal-pseudo-element-handler p)))
 
+(define internal-uri-import-handler (lambda (uri) (open-input-string "")))
+(define scss:set-uri-import-handler!
+  (lambda (p)
+    (verify-arg-types "scss:set-uri-import-handler!"
+		      (list procedure?)
+		      (list p)
+		      1)
+    (set! internal-uri-import-handler p)))
+
 (define null-merge (lambda (x y) (if (not (null? x)) (cons x y) y)))
 
 (define merge-selectors
@@ -538,12 +559,20 @@
 
 (define pseudo-class? 
   (lambda (str) 
-    (member str '(":first-child" ":link" ":visited" ":hover" ":active" ":focus"
-		  ":lang"))))
+    (or (equal? str ":first-child")
+	(equal? str ":link")
+	(equal? str ":visited")
+	(equal? str ":hover")
+	(equal? str ":active")
+	(equal? str ":focus")
+	(equal? str ":lang"))))
 
 (define pseudo-element? 
   (lambda (str)
-    (member str '(":first-line" ":first-letter" ":before" ":after"))))
+    (or (equal? str ":first-line")
+	(equal? str ":first-letter")
+	(equal? str ":before")
+	(equal? str ":after"))))
 
 (define validate-selector 
   (lambda (sel)
@@ -564,8 +593,9 @@
 					     (pseudo-element? y)))
 			 (exact-exec id-regexc y)
 			 (exact-exec dot-regexc y)))
-		    (else (error #t (string-append "scss: invalid selector "
-						   (apply string-append x))))))
+		    (else (error #t 
+				 (string-append "scss: invalid selector "
+						(apply string-append x))))))
 	      x)))
        sel))))
 
@@ -598,7 +628,8 @@
     (let ((slist (string-split v #\sp)))
       (if (equal? (car (last-pair slist)) "!important")
 	  (let ((relist (list-head slist (- (length slist) 1))))
-	    (list (fold (lambda (x y) (if (null? y) x (string-append x " " y)))
+	    (list (fold (lambda (x y) 
+			  (if (null? y) x (string-append x " " y)))
 			(list-head relist (- (length relist) 1))
 			(last-pair relist)) 
 		  '!))
@@ -609,7 +640,7 @@
     (let ((pe (find (lambda (l) (exact-match (car l) property)) 
 		    property-table)))
       (if pe 
-	  (if (apply (cadr pe) (list value)) 
+	  (if (apply (cadr pe) (list (string-downcase value))) 
 	      #t 
 	      (error #t (string-append 
 			 "invalid value " value " for property " property)))
@@ -619,15 +650,15 @@
   (lambda (name predicates args startnum)
     (if (not (= (length predicates) (length args)))
 	(error #t "predicate argument mismatch"))
-    (let ((i 0))
-      (for-each 
-       (lambda (x) 
-	 (if (not (apply x (list (list-ref args i))))
-	     (error #t (string-append name 
-				      ": wrong argument type in position "
-				      (number->string (+ startnum i)))))
-	 (set! i (+ i 1)))
-       predicates))))
+    (letrec ((f (lambda (l1 l2 i)
+		  (if (null? l1) 
+		      #t
+		      (if (not (apply (car l1) (list (car l2))))
+			  (error #t (string-append 
+				     name ": wrong argument type in position "
+				     (number->string (+ startnum i))))
+			  (f (cdr l1) (cdr l2) (+ i 1)))))))
+      (f predicates args 0))))
 
 (define display-list (lambda lst (display lst)))
 
@@ -637,29 +668,84 @@
       (if (and (car tok) (not (eqv? (car tok) 0))) tok '(*eoi*)))))
 
 (define scss:css->scss 
-  (lambda (port)
+  (lambda (port . baseurl)
     (verify-arg-types "scss:css->scss" (list port?) (list port) 1)
-    (lexer-init 'port port)
-    (scss-parser lexer-wrapper display-list)))
+    (if (not (null? baseurl)) 
+	(verify-arg-types "scss:css->scss" (list string?) baseurl 1))
+    (let* ((dirstr (lambda (str)
+		     (let ((s (string-rindex str #\/)))
+		       (if s (substring str 0 (+ s 1)) ""))))
+	   (uh (make-regexp "^\\w+\\:\\/\\/"))
+	   (bu (if (null? baseurl)
+		   (if (file-port? port) 
+		       (dirstr (port-filename port))
+		       (getcwd))
+		   baseurl)))
+      (lexer-init 'port port)
+      (let* ((parsed-sheet (scss-parser lexer-wrapper display-list))
+	     (imports (letrec 
+			  ((f (lambda (s i)
+				(if (not (null? s)) 
+				    (if (eq? (caar s) '@import)
+					(f (cdr s) (cons (cadar s) i))
+					i)
+				    i))))
+			(f parsed-sheet (list))))
+	     (clean-sheet (list-tail parsed-sheet (length imports))))
+	(fold-right 
+	 (lambda (s1 s2)
+	   (let* ((s1 (if (and (> (string-length s1) 7)
+			       (equal? (substring s1 0 7) "file://"))
+			  (substring s1 7)
+			  s1))
+		  (nbu (dirstr s1))
+		  (p (false-if-exception 
+		      (cond ((and (> (string-length s1) 0)
+				  (equal? (substring s1 0 1) "/"))
+			     (open-input-file s1))
+			    ((regexp-exec uh s1)) ;; pass it off to a handler?
+			    (else (open-input-file (string-append bu s1))))))
+		  (new-sheet (false-if-exception (scss:css->scss p nbu))))
+	     (if new-sheet (append new-sheet s2) s2)))
+	 clean-sheet
+	 imports)))))
 
 (define scss:scss->css
   (lambda (stylesheet port)
-    (let* ((ser-selector (lambda (selector)
-			   (display (car selector) port)))
-	   (ser-property (lambda (pair) 
-			  (display "\t" port)
-			  (display (car pair) port)
-			  (display ": " port)
-			  (display (cadr pair) port)
-			  (display ";" port)
-			  (newline port)))
-	   (ser-block (lambda (block)
-			(ser-selector (car block))
-			(display " {" port)
-			(newline port)
-			(for-each (lambda (x) (ser-property x)) (cadr block))
-			(display "}" port)
-			(newline port))))			  
+    (let* 
+	((ser-selector 
+	  (lambda (selector)
+	    (display 
+	     (fold (lambda (x y) 
+		     (string-append 
+		      (fold (lambda (q r)
+			      (string-append 
+			       q (if (or (exact-exec ident-regexc r)
+					 (equal? r ">")
+					 (equal? r "+")
+					 (equal? r "*"))
+				     " " "") r))
+			    ""
+			    (reverse x))
+		      (if (null? y) "" (string-append ", " y))))
+		   '()
+		   (reverse selector))
+	     port)))
+	 (ser-property (lambda (pair) 
+			 (display "\t" port)
+			 (display (car pair) port)
+			 (display ": " port)
+			 (display (cadr pair) port)
+			 (display ";" port)
+			 (newline port)))
+	 (ser-block (lambda (block)
+		      (ser-selector (car block))
+		      (display " {" port)
+		      (newline port)
+		      (for-each (lambda (x) (ser-property x)) (cadr block))
+		      (display "}" port)
+		      (newline port)
+		      (newline port))))			  
       (verify-arg-types "scss:scss->css"
 			(list scss:stylesheet? port?) 
 			(list stylesheet port)
@@ -743,8 +829,11 @@
 		       (list? (cadr x))
 		       (every (lambda (y)
 				(and (list? y)
-				     (= (length y) 2)
-				     (every string? y)))
+				     (or (= (length y) 2)
+					 (and (= (length y) 3)
+					      (eq? (list-ref y 2) '!)))
+				     (string? (car y))
+				     (string? (cadr y))))
 			      (cadr x))))
 		stylesheet))))
 
@@ -781,7 +870,9 @@
 (define scss:color->hex 
   (lambda (color)
     (verify-arg-types "scss:color->hex" (list string?) (list color) 1)
-    (find (lambda (x) (equal? (car x) color)) color-table)))
+    (let ((c (find (lambda (x) (equal? (car x) (string-downcase color))) 
+		   color-table)))
+      (if c (cadr c) (list)))))
 
 (define get-prop-entry 
   (lambda (p) (find (lambda (x) (exact-match (car x) p)) property-table)))
@@ -793,35 +884,54 @@
 (define get-ancestors 
   (lambda (doc node)
     (if (not (null? node))
-	(let ((p (get-sxml-parent doc node))) 
+	(let ((p (get-sxml-parent doc node)))
 	  (if (and p (not (null? p))) (cons p (get-ancestors doc p)) (list)))
 	(list))))
 
 (define expand-stylesheet
   (lambda (stylesheet)
     (let ((ret (list)))
-      (for-each (lambda (x)
-		  (for-each (lambda (y)
-			      (if (not (null? (cdr x)))
-				  (set! ret (append ret 
-						    (list (cons y (cdr x)))))))
-			    (car x)))
+      (for-each 
+       (lambda (x)
+	 (for-each (lambda (y)
+		     (if (not (null? (cdr x)))
+			 (set! ret (append ret (list (cons y (cdr x)))))))
+		   (car x)))
 		stylesheet)
       ret)))
 
+;; This function is a bit complicated.  The general idea is that for certain
+;; properties, if they're not explicitly defined in the matched selector 
+;; block, you can derive their value by looking at a more general property 
+;; that is defined in that block.
+
 (define get-general-property
   (lambda (prop pvlist)
     (let* ((split-matches 
 	    (lambda (pattern str)
 	      (let* ((lst (list))
 		     (f (lambda (x) 
-			  (set! lst (append lst (list (match:substring x)))))))
-		(regexp-substitute/global #f pattern str f)
+			  (set! lst (append 
+				     lst (list (match:substring x)))))))
+		(regexp-substitute/global #f pattern str f 'post)
 		lst)))
-	   (f (lambda (p) (find (lambda (x) (equal? p (car x))) pvlist)))
+
+	   ;; We reverse the property-value list here because we're looking 
+	   ;; for the last-specified value.  Order matters!
+
+	   (f (lambda (p) 
+		(let ((r (find (lambda (x) 
+				 (equal? p (string-downcase (car x)))) 
+			       (reverse pvlist))))
+		  (if r (cadr r) r))))
 	   (prop-bits (string-split prop #\-))
+
+	   ;; The way values are mapped to specific properties depends on the 
+	   ;; number of values as per 
+	   ;; http://www.w3.org/TR/CSS21/box.html#propdef-border-top-color
+
 	   (g (lambda (pat d x)
-		(let ((ml (split-matches pat (cadr x))))
+		(let ((ml (split-matches pat x)))
 		  (cond ((= (length ml) 1) (car ml)) 
 			((= (length ml) 2)
 			 (if (member d '("top" "bottom")) (car ml) (cadr ml)))
@@ -831,8 +941,10 @@
 			       (else (caddr ml))))
 			((= (length ml) 4)
 			 (list-ref 
-			  ml (list-index '("top" "right" "bottom" "left") d)))
+			  ml (list-index (lambda (y) (equal? y d))
+					 '("top" "right" "bottom" "left"))))
 			(else (list))))))
+
 	   (h (lambda (str pattern)
 		(let ((match (if (string? pattern) 
 				 (string-match pattern str)
@@ -842,26 +954,56 @@
 	    ((equal? (car prop-bits) "border")
 	     (if (= (length prop-bits) 2)
 		 (let ((b (f "border"))) 
-		   (if b (g (string-append "(" 
-					   border-width-regex "|" 
-					   border-style-regex "|" 
-					   pcolor-regex 
-					   ")")
-			    (cadr prop-bits) 
-			    b) 
+		   (if b 
+		       (cond ((equal? (cadr prop-bits) "color") 
+			      (h b color-regex))
+			     ((equal? (cadr prop-bits) "width")
+			      (h b border-width-regex))
+			     ((equal? (cadr prop-bits) "style")
+			      (h b border-style-regex))
+
+			     ;; Else it's one of top, left, bottom, right
+
+			     (else b))
 		       (list)))
-		 (let ((bd (get-general-property 
-			    (string-append (car prop-bits) "-" 
-					   (cadr prop-bits))
-			    pvlist)))
-		   (if (not (null? bd))
-		       (cond ((equal? (caddr prop-bits) "color")
-			      (h bd color-regex))
-			     ((equal? (caddr prop-bits) "width")
-			      (h bd border-width-regex))
-			     ((equal? (caddr prop-bits) "style")
-			      (h bd border-style-regex)))
-		       bd))))
+		 (let* ((d (string-append (car prop-bits) 
+					  "-" (cadr prop-bits)))
+			(a (let ((c (caddr prop-bits)))
+			     (if (equal? c "color")
+				 (list-index (lambda (x) 
+					       (or (equal? (car x)
+							   "border-color")
+						   (equal? (car x) "color")))
+					     (reverse pvlist))
+				 (list-index (lambda (x) 
+					       (equal? (car x) 
+						       (string-append 
+							"border-" c)))
+					     (reverse pvlist)))))
+			(b (list-index (lambda (x) (equal? (car x) d))
+				       (reverse pvlist)))
+			(a (if (and a b (> a b)) 
+			       #f (if a 
+				      (cadr (list-ref (reverse pvlist) a)) 
+				      a)))
+			(b (if (and a b (> b a)) 
+			       #f (if b 
+				      (cadr (list-ref (reverse pvlist) b)) 
+				      b))))
+		   (cond (a (cond ((equal? (caddr prop-bits) "color")
+				   (g color-regex (cadr prop-bits) a))
+				  ((equal? (caddr prop-bits) "width")
+				   (g border-width-regex (cadr prop-bits) a))
+				  ((equal? (caddr prop-bits) "style")
+				   (g border-style-regex (cadr prop-bits) 
+				      a))))
+			 (b (cond ((equal? (caddr prop-bits) "color")
+				   (h b color-regex))
+				  ((equal? (caddr prop-bits) "width")
+				   (h b border-width-regex))
+				  ((equal? (caddr prop-bits) "style")
+				   (h b border-style-regex))))
+			 (else (get-general-property d pvlist))))))
 	    ((equal? (car prop-bits) "padding")
 	     (let ((p (f "padding"))) 
 	       (if p (g padding-regex (cadr prop-bits) p) (list))))
@@ -870,53 +1012,69 @@
 	       (if m (g margin-regex (cadr prop-bits) m) (list))))
 	    ((equal? (car prop-bits) "background")
 	     (let ((b (f "background")))
-	       (cond ((equal? (cadr prop-bits) "color") (h b pcolor-regex))
-		     ((equal? (cadr prop-bits) "attach") (h b pattach-regex))
-		     ((equal? (cadr prop-bits) "image") (h b pimage-regex))
-		     ((equal? (cadr prop-bits) "repeat") (h b prepeat-regex))
-;; FIX THIS!!!
-		     ((equal? (cadr prop-bits) "position"))
-		     (else (list)))))
+	       (if b 
+		   (cond ((equal? (cadr prop-bits) "color") 
+			  (h b pcolor-regex))
+			 ((equal? (cadr prop-bits) "attach") 
+			  (h b pattach-regex))
+			 ((equal? (cadr prop-bits) "image") 
+			  (h b pimage-regex))
+			 ((equal? (cadr prop-bits) "repeat") 
+			  (h b prepeat-regex))
+			 ;; FIX THIS!!!
+			 ((equal? (cadr prop-bits) "position"))
+			 (else (list)))
+		   (list))))
 	    ((and (equal? (car prop-bits) "list")
 		  (equal? (cadr prop-bits) "style"))
 	     (let ((l (f "list-style")))
-	       (cond ((equal? (caddr prop-bits) "position")
-		      (h l (string-append lsposition-regex "|(inherit)")))
-		     ((equal? (caddr prop-bits) "image")
-		      (h l (string-append lsimage-regex "|(inherit)")))
-		     ((equal? (caddr prop-bits) "type")
-		      (h l (string-append lstype-regex "|(inherit)")))
-		     (else (list)))))
+	       (if l
+		   (cond ((equal? (caddr prop-bits) "position")
+			  (h l (string-append lsposition-regex "|(inherit)")))
+			 ((equal? (caddr prop-bits) "image")
+			  (h l (string-append lsimage-regex "|(inherit)")))
+			 ((equal? (caddr prop-bits) "type")
+			  (h l (string-append lstype-regex "|(inherit)")))
+			 (else (list)))
+		   (list))))
 	    ((equal? (car prop-bits) "outline")
 	     (let ((o (f "outline")))
-	       (cond ((equal? (cadr prop-bits) "color")
-		      (h o (string-append ocolor-regex "|(inherit)")))
-		     ((equal? (cadr prop-bits) "style")
-		      (h o (string-append ostyle-regex "|(inherit)")))
-		     ((equal? (cadr prop-bits) "width")
-		      (h o (string-append owidth-regex "|(inherit)")))
-		     (else (list)))))
+	       (if o
+		   (cond ((equal? (cadr prop-bits) "color")
+			  (h o (string-append ocolor-regex "|(inherit)")))
+			 ((equal? (cadr prop-bits) "style")
+			  (h o (string-append ostyle-regex "|(inherit)")))
+			 ((equal? (cadr prop-bits) "width")
+			  (h o (string-append owidth-regex "|(inherit)")))
+			 (else (list)))
+		   (list))))
 	    ((equal? (car prop-bits) "font")
-	     (let ((font (f "font"))) 
-	       (cond ((equal? (cadr prop-bits) "family") 
-		      (h font ffamily-regex))
-		     ((equal? (cadr prop-bits) "size") (h font fsize-regex))
-		     ((equal? (cadr prop-bits) "style") (h font fstyle-regex))
-		     ((equal? (cadr prop-bits) "variant")
-		      (h font fvariant-regex))
-		     ((equal? (cadr prop-bits) "weight") 
-		      (h font fweight-regex))
-		     (else (list)))))	    
+	     (let ((font (f "font")))
+	       (if font
+		   (cond ((equal? (cadr prop-bits) "family") 
+			  (h font ffamily-regex))
+			 ((equal? (cadr prop-bits) "size") 
+			  (h font fsize-regex))
+			 ((equal? (cadr prop-bits) "style") 
+			  (h font fstyle-regex))
+			 ((equal? (cadr prop-bits) "variant")
+			  (h font fvariant-regex))
+			 ((equal? (cadr prop-bits) "weight") 
+			  (h font fweight-regex))
+			 (else (list)))
+		   (list))))
 	    (else (list))))))
 
-;; This method is really just searching for selectors that contain the required
-;; property...
+;; This method is really just searching for selectors that contain the 
+;; required property...
 
 (define expand-properties
   (lambda (prop expanded-stylesheet s)
     (let ((ret (list)))
       (for-each (lambda (x) 
-		  (let ((p (find (lambda (y) (equal? (car y) prop)) (cadr x))))
+		  (let ((p (find (lambda (y) 
+				   (equal? (string-downcase (car y)) prop)) 
+				 (cadr x))))
 		    (if (not p) 
 			(let ((q (get-general-property prop (cadr x))))
 			  (if (not (null? q))
@@ -927,8 +1085,7 @@
 
 (define lookup-value
   (lambda (cascade selector prop)
-    (let* ((g (lambda (y) (equal? (car y) prop)))
-	   (ge (lambda (y) (equal? (caaddr y) prop)))
+    (let* ((ge (lambda (y) (equal? (string-downcase (caaddr y)) prop)))
 	   (h (lambda (x) (and (= (length (car x)) 1)
 			       (or (equal? (caar x) selector)
 				   (equal? (caar x) "*")))))
@@ -959,81 +1116,87 @@
   (lambda (cascade node doc prop)
     (letrec 
 	((f (lambda (sel nodes)
-	      (let ((ni (letrec 
-			    ((next-ident 
-			      (lambda (sel-list)
-				(if (not (null? sel-list)) 
-				    (if (exact-exec ident-regexc
-						    (car sel-list)) 
-					(car sel-list)
-					(next-ident (cdr sel-list)))
-				    '()))))
-			  (next-ident sel))))
-		(cond ((null? sel) #t)
-		      ((null? nodes) #f)
-		      ((equal? (car sel) ":first-child")
-		       (if (>= (length nodes) 2)
-			   (let ((cl (find (lambda (x) 
-					     (and (list x) 
-						  (not (eq? (car x) '@))))
-					   (cadr nodes))))
-			     (if (eq? (car nodes) (car cl))
-				 (f (cdr sel) nodes)
-				 #f))
-			   #f))
-		      ((pseudo-class? (car sel))
-		       (if (and (equal? (sxml-node-name (car nodes)) ni)
-				(internal-pseudo-class-handler 
-				 (substring (car sel) 1) doc (car nodes)))
-			   (f (cdr sel) nodes)
-			   #f))
-		      ((pseudo-element? (car sel))
-		       (if (and (equal? (sxml-node-name (car nodes)) ni)
-				(internal-pseudo-element-handler 
-				 (substring (car sel) 1) doc (car nodes)))
-			   (f (cdr sel) nodes)
-			   #f))
-		      ((equal? (substring (car sel) 0 1) "#")
-		       (if (and (equal? (sxml-node-name (car nodes)) ni)
-				(internal-id-handler 
-				 (substring (car sel) 1) doc (car nodes)))
-			   (f (cdr sel) nodes)
-			   #f))
-		      ((equal? (substring (car sel) 0 1) ".")
-		       (letrec ((g (lambda (x) 
-				     (if (and (not (null? x))
-					      (equal? (substring (car x) 0 1)
-						      "."))
-					 (cons (substring (car x) 1) 
-					       (g (cdr x)))
-					 '())))
-				(l (cons (substring (car sel) 1) 
-					 (g (cdr sel)))))
-			 (if (and (equal? (sxml-node-name (car nodes)) ni)
-				  (internal-dot-handler l doc (car nodes)))
-			     (f (list-cdr-ref sel (length l)) nodes)
-			     #f)))
-		      ((equal? (substring (car sel) 0 1) "[")
-		       (let* ((match (regexp-exec attr-sel-regexc 
-						  (car sel)))
-			      (l (match:substring match 1))
-			      (t (match:substring match 15))
-			      (r (match:substring match 16))
-			      (v (sxml-attr-val node l)))
-			 (if (and (string? r)
-				  (or (and (equal? (substring r 0 1) "\"")
-					   (equal? (substring 
-						    r (- (string-length r) 1))
-						   "\""))
-				      (and (equal? (substring r 0 1) "'")
-					   (equal? (substring 
-						    r (- (string-length r) 1))
-						   "'"))))
-			     (set! r (substring r 1 (- (string-length r) 1))))
-			 (if (and v (or (and (equal? t "=")
+	      (letrec 
+		  ((next-ident (lambda (sel-list)
+				 (if (not (null? sel-list)) 
+				     (if (exact-exec ident-regexc
+						     (car sel-list)) 
+					 (car sel-list)
+					 (next-ident (cdr sel-list)))
+				     "*"))))
+		(let ((snn (if (null? nodes) '() (sxml-node-name (car nodes))))
+		      (ni (next-ident sel)))
+		  (cond ((null? sel) #t)
+			((null? snn) #f)
+			((equal? (car sel) ":first-child")
+			 (if (>= (length nodes) 2)
+			     (let ((cl (find (lambda (x) 
+					       (and (list x) 
+						    (not (eq? (car x) '@))))
+					     (cdadr nodes))))
+			       (if (and cl (eq? (car nodes) (car cl)))
+				   (f (cdr sel) nodes)
+				   #f))
+			     #f))
+			((string? (caar nodes)) #f)
+			((pseudo-class? (car sel))
+			 (if (and (or (equal? ni "*") (equal? snn ni))
+				  (internal-pseudo-class-handler 
+				   (substring (car sel) 1) doc (car nodes)))
+			     (f (cdr sel) nodes)
+			     #f))
+			((pseudo-element? (car sel))
+			 (if (and (or (equal? ni "*") (equal? snn ni))
+				  (internal-pseudo-element-handler 
+				   (substring (car sel) 1) doc (car nodes)))
+			     (f (cdr sel) nodes)
+			     #f))
+			((equal? (substring (car sel) 0 1) "#")
+			 (if (and (or (equal? ni "*") (equal? snn ni))
+				  (internal-id-handler 
+				   (substring (car sel) 1) doc (car nodes)))
+			     (f (cdr sel) nodes)
+			     #f))
+			((equal? (substring (car sel) 0 1) ".")
+			 (letrec ((g (lambda (x) 
+				       (if (and (not (null? x))
+						(equal? (substring (car x) 0 1)
+							"."))
+					   (cons (substring (car x) 1) 
+						 (g (cdr x)))
+					   '()))))
+			   (let ((l (cons (substring (car sel) 1) 
+					  (g (cdr sel)))))
+			     (if (and (or (equal? ni "*") (equal? snn ni))
+				      (internal-dot-handler l doc (car nodes)))
+				 (f (list-cdr-ref sel (length l)) nodes)
+				 #f))))
+			((equal? (substring (car sel) 0 1) "[")
+			 (let* ((match (regexp-exec attr-sel-regexc 
+						    (car sel)))
+				(l (match:substring match 1))
+				(t (match:substring match 15))
+				(r (match:substring match 16))
+				(v (sxml-attr-val node l)))
+			   (if (and (string? r)
+				    (or (and (equal? (substring r 0 1) "\"")
+					     (equal? (substring 
+						      r (- (string-length r) 
+							   1))
+						     "\""))
+					(and (equal? (substring r 0 1) "'")
+					     (equal? (substring 
+						      r (- (string-length r) 
+							   1))
+						     "'"))))
+			       (set! r (substring r 1 (- (string-length r) 
+							 1))))
+			   (if (and v 
+				    (or (and (equal? t "=")
 					     (equal? (cadr v) 
 						     (substring 
-						      r 1 (- (length r) 2))))
+						      r 1 (- (string-length r)
+							     2))))
 					(and (equal? t "~=")
 					     (find (lambda (x) (equal? x r))
 						   (string-split (cadr v) 
@@ -1048,24 +1211,23 @@
 							 1))
 						     (string-append r "-")))
 					(not t)))
-			     (f (cdr sel) nodes)
-			     #f)))
-		      ((equal? (car sel) (sxml-node-name (car nodes)))
-		       (let ((i (list-index 
-				 (lambda (x) 
-				   (equal? (sxml-node-name x) ni)) 
-				 (cdr nodes))))
-			 (cond ((null? (cdr sel)) #t)
-			       (i (f (cdr sel) (list-cdr-ref nodes i)))
-			       (else #f))))
-		      ((equal? (car sel) "*") (f (cdr sel) (cdr nodes)))
-		      (else #f))))))
+			       (f (cdr sel) nodes)
+			       #f)))
+			((equal? (car sel) snn)
+			 (let ((i (list-index 
+				   (lambda (x)
+				     (equal? (sxml-node-name x) 
+					     (next-ident (cdr sel))))
+				   (cdr nodes))))
+			   (cond ((null? (cdr sel)) #t)
+				 (i (f (cdr sel) (list-cdr-ref (cdr nodes) i)))
+				 (else #f))))
+			((equal? (car sel) "*") (f (cdr sel) (cdr nodes)))
+			(else #f)))))))
       (let* ((a (cons node (get-ancestors doc node)))
-	     (g (lambda (y) (equal? (car y) prop)))
-	     (ge (lambda (y) (equal? (caaddr y) prop)))
+	     (ge (lambda (y) (equal? (string-downcase (caaddr y)) prop)))
 	     (h (lambda (x) (and (f (reverse (car x)) a) 
-				 (not (null? (cdr x)))
-				 (find g (cadr x)))))
+				 (not (null? (cdr x))))))
 	     (agent-matches (if (not (null? (car cascade)))
 				(filter h (expand-stylesheet (car cascade)))
 				(list)))
@@ -1089,12 +1251,20 @@
 	    (cadr (list-ref (car sr) 2))
 	    sr)))))
 
-;; This is a hack -- need to fix this for cases where default val depends on
-;; other defaults / lookups.
-
 (define get-default-prop-value
   (lambda (pe) 
-    (if (or (not pe) (null? pe)) '() (car (list-ref pe 2)))))
+    (if (or (not pe) (null? pe))
+	(list)
+	(fold (lambda (x y)
+		(let ((z (if (equal? y "") "" (string-append " " y)))) 
+		  (if (symbol? x)
+		      (let ((r (get-default-prop-value 
+				(get-prop-entry 
+				 (symbol->string x)))))
+			(if (and r (not (null? r))) (string-append r z) z))
+		      (string-append x z))))
+	      ""
+	      (list-ref pe 2)))))
 
 (define scss:select-value
   (lambda (source selector . r)
@@ -1121,27 +1291,29 @@
 					 source 
 					 (list (list) (list) source))
 				     selector
-				     (car r))))
+				     (string-downcase (car r)))))
 	      (if (null? val)
 		  (get-default-prop-value (get-prop-entry (car r)))
-		  val)))
+		  (string-downcase val))))
 	  (begin
 	    (verify-arg-types "scss:select-value" (list sxml-doc? string?) r 3)
-	    (let ((pe (get-prop-entry (cadr r))))
+	    (let ((pe (get-prop-entry (string-downcase (cadr r)))))
 	      (letrec ((f (lambda (c n d p) 
 			    (let ((v (select-value c n d p))
 				  (par (get-sxml-parent d n)))
-			      (if (or (and (null? v) 
-					   par
-					   pe
-					   (list-ref pe 3))
-				      (equal? v "inherit"))
-				  (f c par d p) 
+			      (if (and (and par (not (null? par)))
+				       (or (and (null? v) 
+						pe
+						(list-ref pe 3))
+					   (equal? v "inherit")))
+				  (if (and pe (string? (list-ref pe 3)))
+				      (f c par d (list-ref pe 3))
+				      (f c par d p))
 				  v)))))
 		(let ((val (f (if cascade source (list (list) (list) source))
 			      selector
 			      (car r)
-			      (cadr r))))
+			      (string-downcase (cadr r)))))
 		  (if (null? val) 
 		      (get-default-prop-value pe)
-		      val)))))))))
+		      (string-downcase val))))))))))
